﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace PomTray
{
	public partial class Form1 : Form
	{
		private NotifyIcon trayIcon;
		private ContextMenu trayMenu;
		private System.Threading.Timer tmr;

		const int WORKTIME = 50;
		const int RESTTIME = 10;

		public Form1()
		{
			InitializeComponent();

			trayMenu = new ContextMenu();
			trayMenu.MenuItems.Add(String.Format("Start {0}/{1}", WORKTIME, RESTTIME), OnStart);
			trayMenu.MenuItems.Add("Clear", OnClear);
			trayMenu.MenuItems.Add("Exit", OnExit);

			trayIcon = new NotifyIcon();
			trayIcon.Text = "PomTray";

			trayIcon.ContextMenu = trayMenu;
			trayIcon.Visible = true;

			/* create timer but don't start */
			tmr = new System.Threading.Timer(OnTimer, null,
				System.Threading.Timeout.Infinite,
				System.Threading.Timeout.Infinite);

			OnClear(null, null);
		}

		protected override void OnLoad(System.EventArgs e)
		{
			Visible = false;
			ShowInTaskbar = false;
			base.OnLoad(e);
		}

		int MinutesLeft = 0;

		void OnStart(object sender, EventArgs e)
		{
			/* start timer at 1 minute intervals */
			MinutesLeft = WORKTIME + RESTTIME;
			tmr.Change(1000 * 60, 1000 * 60);

			// for debugging:
			//MinutesLeft = 10;
			//tmr.Change(1000, 1000);

			UpdateIcon(MinutesLeft.ToString(), Color.White, false);
		}

		void OnClear(object sender, EventArgs e)
		{
			tmr.Change(
				System.Threading.Timeout.Infinite,
				System.Threading.Timeout.Infinite);
			UpdateIcon("--", Color.Gray, false);
		}

		void OnTimer(object o)
		{
			MinutesLeft--;

			if (MinutesLeft == 0)
			{
				OnClear(null, null);
			}

			/* time to rest */
			else if (MinutesLeft <= RESTTIME)
			{
				UpdateIcon(MinutesLeft.ToString(), Color.Red, true);
			}

			/* if below 15 or evenly divisible by 5, draw in white */
			else if (MinutesLeft <= 15 || MinutesLeft % 5 == 0)
			{
				UpdateIcon(MinutesLeft.ToString(), Color.White, false);
			}

			if (MinutesLeft == RESTTIME)
			{
				System.Media.SystemSounds.Asterisk.Play();
				MessageBox.Show("Finish Up / Take a Break", "PomTray", 
					MessageBoxButtons.OK,
					MessageBoxIcon.None,
					MessageBoxDefaultButton.Button1,
					MessageBoxOptions.DefaultDesktopOnly);
			}

			else if (MinutesLeft == 0)
			{
				System.Media.SystemSounds.Exclamation.Play();
				MessageBox.Show("Break's Over, Get Back to Work", "PomTray",
					MessageBoxButtons.OK,
					MessageBoxIcon.None,
					MessageBoxDefaultButton.Button1,
					MessageBoxOptions.DefaultDesktopOnly);
			}
		}

		void UpdateIcon(String text, Color c, bool bold)
		{
			Brush brush = new SolidBrush(c);
			Bitmap bitmap = new Bitmap(16, 16);
			Graphics graphics = Graphics.FromImage(bitmap);
			Font font = new Font(FontFamily.GenericSansSerif, 10.0f, bold ? FontStyle.Bold : FontStyle.Regular);
			graphics.DrawString(text, font, brush, 0,0);
			IntPtr hIcon = bitmap.GetHicon();
			Icon icon = Icon.FromHandle(hIcon);
			trayIcon.Icon = icon;
		}

		void OnExit(object sender, EventArgs e)
		{
			trayIcon.Visible = false;
			Application.Exit();
		}
	}
}
